@@ -29,43 +29,6 @@
 #include "pgp/seahorse-gpgme-uid.h"
 #include "pgp/seahorse-gpgme-photo.h"
 
-/*
- * Key type options.
- * We only support GPG version >=2.0.12 or >= 2.1.4
- */
-
-typedef enum {
-	RSA_RSA = 1,
-	DSA_ELGAMAL = 2,
-	DSA = 3,
-	RSA_SIGN = 4,
-	ELGAMAL = 5,
-	RSA_ENCRYPT = 6
-} SeahorseKeyEncType;
-
-/* Length ranges for key types */
-typedef enum {
-	/* Minimum length for #DSA. */
-	DSA_MIN = 768,
-	/* Maximum length for #DSA. */
-#if ( GPG_MAJOR == 2 &&   GPG_MINOR == 0 && GPG_MICRO < 12 ) || \
-    ( GPG_MAJOR == 1 && ( GPG_MINOR <  4 || GPG_MICRO < 10 ) )
-	DSA_MAX = 1024,
-#else
-	DSA_MAX = 3072,
-#endif
-	/* Minimum length for #ELGAMAL. Maximum length is #LENGTH_MAX. */
-	ELGAMAL_MIN = 768,
-	/* Minimum length of #RSA_SIGN and #RSA_ENCRYPT. Maximum length is
-	 * #LENGTH_MAX.
-	 */
-	RSA_MIN = 1024,
-	/* Maximum length for #ELGAMAL, #RSA_SIGN, and #RSA_ENCRYPT. */
-	LENGTH_MAX = 4096,
-	/* Default length for #ELGAMAL, #RSA_SIGN, #RSA_ENCRYPT, and #DSA. */
-	LENGTH_DEFAULT = 2048
-} SeahorseKeyLength;
-
 typedef enum {
 	/* Unknown key check */
 	SIGN_CHECK_NO_ANSWER,
@@ -170,11 +133,18 @@ gboolean          seahorse_gpgme_key_op_make_primary_finish (SeahorseGpgmeUid *u
                                                              GError **error);
 
 gpgme_error_t         seahorse_gpgme_key_op_del_uid          (SeahorseGpgmeUid *uid);
-                             
-gpgme_error_t         seahorse_gpgme_key_op_add_subkey       (SeahorseGpgmeKey *pkey,
-                                                              SeahorseKeyEncType type,
-                                                              guint length,
-                                                              time_t expires);
+
+void              seahorse_gpgme_key_op_add_subkey_async    (SeahorseGpgmeKey *pkey,
+                                                             SeahorseKeyEncType type,
+                                                             guint length,
+                                                             gulong expires,
+                                                             GCancellable *cancellable,
+                                                             GAsyncReadyCallback callback,
+                                                             gpointer user_data);
+
+gboolean          seahorse_gpgme_key_op_add_subkey_finish   (SeahorseGpgmeKey *pkey,
+                                                             GAsyncResult *result,
+                                                             GError **error);
 
 gpgme_error_t         seahorse_gpgme_key_op_del_subkey       (SeahorseGpgmeSubkey *subkey);
 
