@@ -47,14 +47,17 @@
 #include <locale.h>
 
 enum {
-	PROP_0,
-	PROP_LABEL,
-	PROP_DESCRIPTION,
-	PROP_ICON,
-	PROP_URI,
+    PROP_0,
+    PROP_LABEL,
+    PROP_DESCRIPTION,
+    PROP_ICON,
+    PROP_CATEGORY,
+    PROP_URI,
     PROP_ACTIONS,
     PROP_ACTION_PREFIX,
-    PROP_MENU_MODEL
+    PROP_MENU_MODEL,
+    PROP_SHOW_IF_EMPTY,
+    N_PROPS
 };
 
 /* Amount of keys to load in a batch */
@@ -564,8 +567,8 @@ keyring_import_free (gpointer data)
 
 static void
 on_keyring_import_loaded (GObject *source,
-                         GAsyncResult *result,
-                         gpointer user_data)
+                          GAsyncResult *result,
+                          gpointer user_data)
 {
 	g_autoptr(GTask) task = G_TASK (user_data);
 	keyring_import_closure *closure = g_task_get_task_data (task);
@@ -585,7 +588,7 @@ on_keyring_import_loaded (GObject *source,
 		keys = g_list_prepend (keys, object);
 	}
 
-	seahorse_progress_end (g_task_get_cancellable (task), result);
+	seahorse_progress_end (g_task_get_cancellable (task), task);
 	g_task_return_pointer (task, g_steal_pointer (&keys), (GDestroyNotify) g_list_free);
 }
 
@@ -775,6 +778,12 @@ seahorse_gpgme_keyring_get_icon (SeahorsePlace *place)
 	return g_themed_icon_new (GCR_ICON_GNUPG);
 }
 
+static SeahorsePlaceCategory
+seahorse_gpgme_keyring_get_category (SeahorsePlace *place)
+{
+    return SEAHORSE_PLACE_CATEGORY_KEYS;
+}
+
 static GActionGroup *
 seahorse_gpgme_keyring_get_actions (SeahorsePlace *place)
 {
@@ -802,6 +811,12 @@ seahorse_gpgme_keyring_get_uri (SeahorsePlace *place)
 	return g_strdup ("gnupg://");
 }
 
+static gboolean
+seahorse_gpgme_keyring_get_show_if_empty (SeahorsePlace *place)
+{
+    return TRUE;
+}
+
 static void
 seahorse_gpgme_keyring_get_property (GObject *obj,
                                      guint prop_id,
@@ -817,9 +832,12 @@ seahorse_gpgme_keyring_get_property (GObject *obj,
 	case PROP_DESCRIPTION:
 		g_value_take_string (value, seahorse_gpgme_keyring_get_description (place));
 		break;
-	case PROP_ICON:
-		g_value_take_object (value, seahorse_gpgme_keyring_get_icon (place));
-		break;
+    case PROP_ICON:
+        g_value_take_object (value, seahorse_gpgme_keyring_get_icon (place));
+        break;
+    case PROP_CATEGORY:
+        g_value_set_enum (value, seahorse_gpgme_keyring_get_category (place));
+        break;
 	case PROP_URI:
 		g_value_take_string (value, seahorse_gpgme_keyring_get_uri (place));
 		break;
@@ -832,10 +850,13 @@ seahorse_gpgme_keyring_get_property (GObject *obj,
 	case PROP_MENU_MODEL:
 		g_value_take_object (value, seahorse_gpgme_keyring_get_menu_model (place));
 		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
-		break;
-	}
+    case PROP_SHOW_IF_EMPTY:
+        g_value_set_boolean (value, TRUE);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+        break;
+    }
 }
 
 static void
@@ -911,9 +932,11 @@ seahorse_gpgme_keyring_class_init (SeahorseGpgmeKeyringClass *klass)
 	g_object_class_override_property (gobject_class, PROP_DESCRIPTION, "description");
 	g_object_class_override_property (gobject_class, PROP_URI, "uri");
 	g_object_class_override_property (gobject_class, PROP_ICON, "icon");
+    g_object_class_override_property (gobject_class, PROP_CATEGORY, "category");
     g_object_class_override_property (gobject_class, PROP_ACTIONS, "actions");
     g_object_class_override_property (gobject_class, PROP_ACTION_PREFIX, "action-prefix");
     g_object_class_override_property (gobject_class, PROP_MENU_MODEL, "menu-model");
+    g_object_class_override_property (gobject_class, PROP_SHOW_IF_EMPTY, "show-if-empty");
 }
 
 static void
@@ -925,10 +948,12 @@ seahorse_gpgme_keyring_place_iface (SeahorsePlaceIface *iface)
     iface->get_action_prefix = seahorse_gpgme_keyring_get_action_prefix;
     iface->get_menu_model = seahorse_gpgme_keyring_get_menu_model;
 	iface->get_description = seahorse_gpgme_keyring_get_description;
-	iface->get_icon = seahorse_gpgme_keyring_get_icon;
+    iface->get_icon = seahorse_gpgme_keyring_get_icon;
+    iface->get_category = seahorse_gpgme_keyring_get_category;
 	iface->get_label = seahorse_gpgme_keyring_get_label;
 	iface->set_label = seahorse_gpgme_keyring_set_label;
 	iface->get_uri = seahorse_gpgme_keyring_get_uri;
+    iface->get_show_if_empty = seahorse_gpgme_keyring_get_show_if_empty;
 }
 
 static guint
