@@ -18,10 +18,10 @@
  */
 
 /**
- * SeahorseHKPSource: A key source which searches HTTP PGP key servers. 
- * 
+ * SeahorseHKPSource: A key source which searches HTTP PGP key servers.
+ *
  * - Derived from SeahorseServerSource.
- * - Adds found keys to SeahorseContext. 
+ * - Adds found keys to SeahorseContext.
  * - Used by SeahorseServiceDiscovery for retrieving shared keys.
  */
 
@@ -37,9 +37,13 @@ G_DECLARE_FINAL_TYPE (SeahorseHKPSource, seahorse_hkp_source,
                       SEAHORSE, HKP_SOURCE,
                       SeahorseServerSource)
 
-SeahorseHKPSource*    seahorse_hkp_source_new      (const gchar *uri, 
+SeahorseHKPSource*    seahorse_hkp_source_new      (const gchar *uri,
                                                     const gchar *host);
 
 gboolean              seahorse_hkp_is_valid_uri    (const gchar *uri);
 
+
+#define HKP_ERROR_DOMAIN (seahorse_hkp_error_quark())
+GQuark            seahorse_hkp_error_quark       (void);
+
 #endif /* WITH_HKP */
