@@ -8,6 +8,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "hdy-swipe-tracker-private.h"
+#include "hdy-navigation-direction.h"
 
 #include <math.h>
 
@@ -19,53 +20,31 @@
 #define VELOCITY_THRESHOLD 0.4
 #define DURATION_MULTIPLIER 3
 #define ANIMATION_BASE_VELOCITY 0.002
-#define DRAG_THRESHOLD_DISTANCE 5
+#define DRAG_THRESHOLD_DISTANCE 16
 
 /**
- * PRIVATE:hdy-swipe-tracker
- * @short_description: Swipe tracker used in #HdyLeaflet and #HdyPaginator
+ * SECTION:hdy-swipe-tracker
+ * @short_description: Swipe tracker used in #HdyCarousel and #HdyLeaflet
  * @title: HdySwipeTracker
- * @See_also: #HdyLeaflet, #HdyPaginator, #HdySwipeable
- * @stability: Private
+ * @See_also: #HdyCarousel, #HdyDeck, #HdyLeaflet, #HdySwipeable
  *
  * The HdySwipeTracker object can be used for implementing widgets with swipe
  * gestures. It supports touch-based swipes, pointer dragging, and touchpad
  * scrolling.
  *
- * The events must be received as early as possible to defer the events to
- * child widgets when needed. Usually this happens naturally, but
- * GtkScrolledWindow receives events on capture phase via a private function.
- * Because of that, implementing widgets must do the same thing, i.e. receive
- * events on capture phase and call hdy_swipe_tracker_captured_event() for
- * each event. This can be done as follows:
- * |[<!-- language="C" -->
- * g_object_set_data (G_OBJECT (self), "captured-event-handler", captured_event_cb);
- * ]|
- * Where captured_event_cb() is:
- * |[<!-- language="C" -->
- * static gboolean
- * captured_event_cb (MyWidget *self, GdkEvent *event)
- * {
- *   return hdy_swipe_tracker_captured_event (self->tracker, event);
- * }
- * ]|
- *
- * NOTE: In GTK4 this can be replaced by a GtkEventControllerLegacy with capture
- * propagation phase.
- *
  * The widgets will probably want to expose #HdySwipeTracker:enabled property.
  * If they expect to use horizontal orientation, #HdySwipeTracker:reversed
  * property can be used for supporting RTL text direction.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 
 typedef enum {
   HDY_SWIPE_TRACKER_STATE_NONE,
-  HDY_SWIPE_TRACKER_STATE_PREPARING,
   HDY_SWIPE_TRACKER_STATE_PENDING,
   HDY_SWIPE_TRACKER_STATE_SCROLLING,
   HDY_SWIPE_TRACKER_STATE_FINISHING,
+  HDY_SWIPE_TRACKER_STATE_REJECTED,
 } HdySwipeTrackerState;
 
 struct _HdySwipeTracker
@@ -78,19 +57,19 @@ struct _HdySwipeTracker
   gboolean allow_mouse_drag;
   GtkOrientation orientation;
 
+  gint start_x;
+  gint start_y;
+  gboolean use_capture_phase;
+
   guint32 prev_time;
   gdouble velocity;
 
   gdouble initial_progress;
   gdouble progress;
   gboolean cancelled;
-  gdouble cancel_progress;
 
   gdouble prev_offset;
-  gdouble distance;
 
-  gdouble *snap_points;
-  gint n_snap_points;
   gboolean is_scrolling;
 
   HdySwipeTrackerState state;
@@ -114,27 +93,32 @@ enum {
 
 static GParamSpec *props[LAST_PROP];
 
+enum {
+  SIGNAL_BEGIN_SWIPE,
+  SIGNAL_UPDATE_SWIPE,
+  SIGNAL_END_SWIPE,
+  SIGNAL_LAST_SIGNAL,
+};
+
+static guint signals[SIGNAL_LAST_SIGNAL];
+
 static void
 reset (HdySwipeTracker *self)
 {
-  if (self->snap_points) {
-    g_free (self->snap_points);
-    self->snap_points = NULL;
-  }
-
   self->state = HDY_SWIPE_TRACKER_STATE_NONE;
 
   self->prev_offset = 0;
-  self->distance = 0;
 
   self->initial_progress = 0;
   self->progress = 0;
 
+  self->start_x = 0;
+  self->start_y = 0;
+  self->use_capture_phase = FALSE;
+
   self->prev_time = 0;
   self->velocity = 0;
 
-  self->cancel_progress = 0;
-
   self->cancelled = FALSE;
 
   if (self->swipeable)
@@ -142,14 +126,46 @@ reset (HdySwipeTracker *self)
 }
 
 static void
-gesture_prepare (HdySwipeTracker *self,
-                 gint             direction)
+get_range (HdySwipeTracker *self,
+           gdouble         *first,
+           gdouble         *last)
 {
+  g_autofree gdouble *points = NULL;
+  gint n;
+
+  points = hdy_swipeable_get_snap_points (self->swipeable, &n);
+
+  *first = points[0];
+  *last = points[n - 1];
+}
+
+static void
+gesture_prepare (HdySwipeTracker        *self,
+                 HdyNavigationDirection  direction,
+                 gboolean                is_drag)
+{
+  GdkRectangle rect;
+
   if (self->state != HDY_SWIPE_TRACKER_STATE_NONE)
     return;
 
-  self->state = HDY_SWIPE_TRACKER_STATE_PREPARING;
-  hdy_swipeable_begin_swipe (self->swipeable, direction, TRUE);
+  hdy_swipeable_get_swipe_area (self->swipeable, direction, is_drag, &rect);
+
+  if (self->start_x < rect.x ||
+      self->start_x >= rect.x + rect.width ||
+      self->start_y < rect.y ||
+      self->start_y >= rect.y + rect.height) {
+    self->state = HDY_SWIPE_TRACKER_STATE_REJECTED;
+
+    return;
+  }
+
+  hdy_swipe_tracker_emit_begin_swipe (self, direction, TRUE);
+
+  self->initial_progress = hdy_swipeable_get_progress (self->swipeable);
+  self->progress = self->initial_progress;
+  self->velocity = 0;
+  self->state = HDY_SWIPE_TRACKER_STATE_PENDING;
 }
 
 static void
@@ -184,8 +200,7 @@ gesture_update (HdySwipeTracker *self,
   if (time != self->prev_time)
     self->velocity = delta / (time - self->prev_time);
 
-  first_point = self->snap_points[0];
-  last_point = self->snap_points[self->n_snap_points - 1];
+  get_range (self, &first_point, &last_point);
 
   progress = self->progress + delta;
   progress = CLAMP (progress, first_point, last_point);
@@ -195,7 +210,7 @@ gesture_update (HdySwipeTracker *self,
 
   self->progress = progress;
 
-  hdy_swipeable_update_swipe (self->swipeable, progress);
+  hdy_swipe_tracker_emit_update_swipe (self, progress);
 
   self->prev_time = time;
 }
@@ -205,47 +220,54 @@ get_closest_snap_points (HdySwipeTracker *self,
                          gdouble         *upper,
                          gdouble         *lower)
 {
-  gint i;
+  gint i, n;
+  gdouble *points;
 
   *upper = 0;
   *lower = 0;
 
-  for (i = 0; i < self->n_snap_points; i++) {
-    if (self->snap_points[i] >= self->progress) {
-      *upper = self->snap_points[i];
+  points = hdy_swipeable_get_snap_points (self->swipeable, &n);
+
+  for (i = 0; i < n; i++) {
+    if (points[i] >= self->progress) {
+      *upper = points[i];
       break;
     }
   }
 
-  for (i = self->n_snap_points - 1; i >= 0; i--) {
-    if (self->snap_points[i] <= self->progress) {
-      *lower = self->snap_points[i];
+  for (i = n - 1; i >= 0; i--) {
+    if (points[i] <= self->progress) {
+      *lower = points[i];
       break;
     }
   }
+
+  g_free (points);
 }
 
 static gdouble
-get_end_progress (HdySwipeTracker *self)
+get_end_progress (HdySwipeTracker *self,
+                  gdouble          distance)
 {
   gdouble upper, lower, middle;
 
   if (self->cancelled)
-    return self->cancel_progress;
+    return hdy_swipeable_get_cancel_progress (self->swipeable);
 
   get_closest_snap_points (self, &upper, &lower);
   middle = (upper + lower) / 2;
 
   if (self->progress > middle)
-    return (self->velocity * self->distance > -VELOCITY_THRESHOLD ||
+    return (self->velocity * distance > -VELOCITY_THRESHOLD ||
             self->initial_progress > upper) ? upper : lower;
 
-  return (self->velocity * self->distance < VELOCITY_THRESHOLD ||
+  return (self->velocity * distance < VELOCITY_THRESHOLD ||
           self->initial_progress < lower) ? lower : upper;
 }
 
 static void
-gesture_end (HdySwipeTracker *self)
+gesture_end (HdySwipeTracker *self,
+             gdouble          distance)
 {
   gdouble end_progress, velocity;
   gint64 duration;
@@ -253,7 +275,7 @@ gesture_end (HdySwipeTracker *self)
   if (self->state == HDY_SWIPE_TRACKER_STATE_NONE)
     return;
 
-  end_progress = get_end_progress (self);
+  end_progress = get_end_progress (self, distance);
 
   velocity = ANIMATION_BASE_VELOCITY;
   if ((end_progress - self->progress) * self->velocity > 0)
@@ -263,7 +285,7 @@ gesture_end (HdySwipeTracker *self)
   if (self->progress != end_progress)
     duration = CLAMP (duration, MIN_ANIMATION_DURATION, MAX_ANIMATION_DURATION);
 
-  hdy_swipeable_end_swipe (self->swipeable, duration, end_progress);
+  hdy_swipe_tracker_emit_end_swipe (self, duration, end_progress);
 
   if (self->cancelled)
     reset (self);
@@ -272,19 +294,18 @@ gesture_end (HdySwipeTracker *self)
 }
 
 static void
-gesture_cancel (HdySwipeTracker *self)
+gesture_cancel (HdySwipeTracker *self,
+                gdouble          distance)
 {
-  if (self->state == HDY_SWIPE_TRACKER_STATE_PREPARING) {
+  if (self->state != HDY_SWIPE_TRACKER_STATE_PENDING &&
+      self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING) {
     reset (self);
+
     return;
   }
 
-  if (self->state != HDY_SWIPE_TRACKER_STATE_PENDING &&
-      self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
-    return;
-
   self->cancelled = TRUE;
-  gesture_end (self);
+  gesture_end (self, distance);
 }
 
 static void
@@ -295,6 +316,9 @@ drag_begin_cb (HdySwipeTracker *self,
 {
   if (self->state != HDY_SWIPE_TRACKER_STATE_NONE)
     gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+  self->start_x = start_x;
+  self->start_y = start_y;
 }
 
 static void
@@ -303,43 +327,50 @@ drag_update_cb (HdySwipeTracker *self,
                 gdouble          offset_y,
                 GtkGestureDrag  *gesture)
 {
-  gdouble offset;
+  gdouble offset, distance;
   gboolean is_vertical, is_offset_vertical;
 
+  distance = hdy_swipeable_get_distance (self->swipeable);
+
   is_vertical = (self->orientation == GTK_ORIENTATION_VERTICAL);
   if (is_vertical)
-    offset = -offset_y / self->distance;
+    offset = -offset_y / distance;
   else
-    offset = -offset_x / self->distance;
+    offset = -offset_x / distance;
 
   if (self->reversed)
     offset = -offset;
 
   is_offset_vertical = (ABS (offset_y) > ABS (offset_x));
 
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
   if (self->state == HDY_SWIPE_TRACKER_STATE_NONE) {
     if (is_vertical == is_offset_vertical)
-      gesture_prepare (self, offset > 0 ? 1 : -1);
+      gesture_prepare (self, offset > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK, TRUE);
     else
       gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
     return;
   }
 
   if (self->state == HDY_SWIPE_TRACKER_STATE_PENDING) {
-    gdouble distance;
+    gdouble drag_distance;
     gdouble first_point, last_point;
     gboolean is_overshooting;
 
-    first_point = self->snap_points[0];
-    last_point = self->snap_points[self->n_snap_points - 1];
+    get_range (self, &first_point, &last_point);
 
-    distance = sqrt (offset_x * offset_x + offset_y * offset_y);
+    drag_distance = sqrt (offset_x * offset_x + offset_y * offset_y);
     is_overshooting = (offset < 0 && self->progress <= first_point) ||
                       (offset > 0 && self->progress >= last_point);
 
-    if (distance >= DRAG_THRESHOLD_DISTANCE) {
+    if (drag_distance >= DRAG_THRESHOLD_DISTANCE) {
       if ((is_vertical == is_offset_vertical) && !is_overshooting) {
         gesture_begin (self);
+        self->prev_offset = offset;
         gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_CLAIMED);
       } else {
         gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
@@ -359,13 +390,24 @@ drag_end_cb (HdySwipeTracker *self,
              gdouble          offset_y,
              GtkGestureDrag  *gesture)
 {
+  gdouble distance;
+
+  distance = hdy_swipeable_get_distance (self->swipeable);
+
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+    reset (self);
+    return;
+  }
+
   if (self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING) {
-    gesture_cancel (self);
+    gesture_cancel (self, distance);
     gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
     return;
   }
 
-  gesture_end (self);
+  gesture_end (self, distance);
 }
 
 static void
@@ -373,20 +415,28 @@ drag_cancel_cb (HdySwipeTracker  *self,
                 GdkEventSequence *sequence,
                 GtkGesture       *gesture)
 {
-  gesture_cancel (self);
+  gdouble distance;
+
+  distance = hdy_swipeable_get_distance (self->swipeable);
+
+  gesture_cancel (self, distance);
   gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
 }
 
 static gboolean
-captured_scroll_event (HdySwipeTracker *self,
-                       GdkEvent        *event)
+handle_scroll_event (HdySwipeTracker *self,
+                     GdkEvent        *event,
+                     gboolean         capture)
 {
   GdkDevice *source_device;
   GdkInputSource input_source;
-  gdouble dx, dy, delta;
+  gdouble dx, dy, delta, distance;
   gboolean is_vertical;
   gboolean is_delta_vertical;
 
+  is_vertical = (self->orientation == GTK_ORIENTATION_VERTICAL);
+  distance = is_vertical ? TOUCHPAD_BASE_DISTANCE_V : TOUCHPAD_BASE_DISTANCE_H;
+
   if (gdk_event_get_scroll_direction (event, NULL))
     return GDK_EVENT_PROPAGATE;
 
@@ -395,8 +445,6 @@ captured_scroll_event (HdySwipeTracker *self,
   if (input_source != GDK_SOURCE_TOUCHPAD)
     return GDK_EVENT_PROPAGATE;
 
-  is_vertical = (self->orientation == GTK_ORIENTATION_VERTICAL);
-
   gdk_event_get_scroll_deltas (event, &dx, &dy);
   delta = is_vertical ? dy : dx;
   if (self->reversed)
@@ -405,7 +453,7 @@ captured_scroll_event (HdySwipeTracker *self,
   is_delta_vertical = (ABS (dy) > ABS (dx));
 
   if (self->is_scrolling) {
-    gesture_cancel (self);
+    gesture_cancel (self, distance);
 
     if (gdk_event_is_scroll_stop_event (event))
       self->is_scrolling = FALSE;
@@ -413,31 +461,40 @@ captured_scroll_event (HdySwipeTracker *self,
     return GDK_EVENT_PROPAGATE;
   }
 
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    if (gdk_event_is_scroll_stop_event (event))
+      reset (self);
+
+    return GDK_EVENT_PROPAGATE;
+  }
+
   if (self->state == HDY_SWIPE_TRACKER_STATE_NONE) {
     if (gdk_event_is_scroll_stop_event (event))
       return GDK_EVENT_PROPAGATE;
 
-    if (is_vertical == is_delta_vertical)
-      gesture_prepare (self, delta > 0 ? 1 : -1);
-    else {
+    if (is_vertical == is_delta_vertical) {
+      if (!capture) {
+        GtkWidget *widget = gtk_get_event_widget (event);
+        gdouble event_x, event_y;
+
+        gdk_event_get_coords (event, &event_x, &event_y);
+        gtk_widget_translate_coordinates (widget, GTK_WIDGET (self->swipeable),
+                                          event_x, event_y,
+                                          &self->start_x, &self->start_y);
+
+        gesture_prepare (self, delta > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK, FALSE);
+      }
+    } else {
       self->is_scrolling = TRUE;
       return GDK_EVENT_PROPAGATE;
     }
   }
 
-  if (self->state == HDY_SWIPE_TRACKER_STATE_PREPARING) {
-    if (gdk_event_is_scroll_stop_event (event))
-      gesture_cancel (self);
-
-    return GDK_EVENT_PROPAGATE;
-  }
-
-  if (self->state == HDY_SWIPE_TRACKER_STATE_PENDING) {
+  if (!capture && self->state == HDY_SWIPE_TRACKER_STATE_PENDING) {
     gboolean is_overshooting;
     gdouble first_point, last_point;
 
-    first_point = self->snap_points[0];
-    last_point = self->snap_points[self->n_snap_points - 1];
+    get_range (self, &first_point, &last_point);
 
     is_overshooting = (delta < 0 && self->progress <= first_point) ||
                       (delta > 0 && self->progress >= last_point);
@@ -445,26 +502,174 @@ captured_scroll_event (HdySwipeTracker *self,
     if ((is_vertical == is_delta_vertical) && !is_overshooting)
       gesture_begin (self);
     else
-      gesture_cancel (self);
+      gesture_cancel (self, distance);
   }
 
   if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
     if (gdk_event_is_scroll_stop_event (event)) {
-      gesture_end (self);
+      gesture_end (self, distance);
     } else {
-      self->distance = is_vertical ? TOUCHPAD_BASE_DISTANCE_V :
-                       TOUCHPAD_BASE_DISTANCE_H;
-      gesture_update (self, delta / self->distance * SCROLL_MULTIPLIER);
+      gesture_update (self, delta / distance * SCROLL_MULTIPLIER);
       return GDK_EVENT_STOP;
     }
   }
 
-  if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING)
+  if (!capture && self->state == HDY_SWIPE_TRACKER_STATE_FINISHING)
     reset (self);
 
   return GDK_EVENT_PROPAGATE;
 }
 
+static gboolean
+is_window_handle (GtkWidget *widget)
+{
+  gboolean window_dragging;
+  GtkWidget *parent, *window, *titlebar;
+
+  gtk_widget_style_get (widget, "window-dragging", &window_dragging, NULL);
+
+  if (window_dragging)
+    return TRUE;
+
+  /* Window titlebar area is always draggable, so check if we're inside. */
+  window = gtk_widget_get_toplevel (widget);
+  if (!GTK_IS_WINDOW (window))
+    return FALSE;
+
+  titlebar = gtk_window_get_titlebar (GTK_WINDOW (window));
+  if (!titlebar)
+    return FALSE;
+
+  parent = widget;
+  while (parent && parent != titlebar)
+    parent = gtk_widget_get_parent (parent);
+
+  return parent == titlebar;
+}
+
+/* HACK: Since we don't have _gtk_widget_consumes_motion(), we can't do a proper
+ * check for whether we can drag from a widget or not. So we trust the widgets
+ * to propagate or stop their events. However, GtkButton stops press events,
+ * making it impossible to drag from it.
+ */
+static gboolean
+should_force_drag (HdySwipeTracker *self,
+                   GtkWidget       *widget)
+{
+  GtkWidget *parent;
+
+  if (!GTK_IS_BUTTON (widget))
+    return FALSE;
+
+  parent = widget;
+  while (parent && !HDY_IS_SWIPEABLE (parent))
+    parent = gtk_widget_get_parent (parent);
+
+  return parent == GTK_WIDGET (self->swipeable);
+}
+
+static gboolean
+handle_event_cb (HdySwipeTracker *self,
+                 GdkEvent        *event)
+{
+  GdkEventSequence *sequence;
+  gboolean retval;
+  GtkEventSequenceState state;
+  GtkWidget *widget;
+
+  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
+    return GDK_EVENT_PROPAGATE;
+
+  if (self->use_capture_phase)
+    return GDK_EVENT_PROPAGATE;
+
+  if (event->type == GDK_SCROLL)
+    return handle_scroll_event (self, event, FALSE);
+
+  if (event->type != GDK_BUTTON_PRESS &&
+      event->type != GDK_BUTTON_RELEASE &&
+      event->type != GDK_MOTION_NOTIFY &&
+      event->type != GDK_TOUCH_BEGIN &&
+      event->type != GDK_TOUCH_END &&
+      event->type != GDK_TOUCH_UPDATE &&
+      event->type != GDK_TOUCH_CANCEL)
+    return GDK_EVENT_PROPAGATE;
+
+  widget = gtk_get_event_widget (event);
+  if (is_window_handle (widget))
+    return GDK_EVENT_PROPAGATE;
+
+  sequence = gdk_event_get_event_sequence (event);
+  retval = gtk_event_controller_handle_event (GTK_EVENT_CONTROLLER (self->touch_gesture), event);
+  state = gtk_gesture_get_sequence_state (self->touch_gesture, sequence);
+
+  if (state == GTK_EVENT_SEQUENCE_DENIED) {
+    gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
+    return GDK_EVENT_PROPAGATE;
+  }
+
+  if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
+    return GDK_EVENT_STOP;
+  } else if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING) {
+    reset (self);
+    return GDK_EVENT_STOP;
+  }
+  return retval;
+}
+
+static gboolean
+captured_event_cb (HdySwipeable *swipeable,
+                   GdkEvent     *event)
+{
+  HdySwipeTracker *self = hdy_swipeable_get_swipe_tracker (swipeable);
+  GtkWidget *widget;
+  GdkEventSequence *sequence;
+  gboolean retval;
+  GtkEventSequenceState state;
+
+  g_assert (HDY_IS_SWIPE_TRACKER (self));
+
+  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
+    return GDK_EVENT_PROPAGATE;
+
+  if (event->type == GDK_SCROLL)
+    return handle_scroll_event (self, event, TRUE);
+
+  if (event->type != GDK_BUTTON_PRESS &&
+      event->type != GDK_BUTTON_RELEASE &&
+      event->type != GDK_MOTION_NOTIFY &&
+      event->type != GDK_TOUCH_BEGIN &&
+      event->type != GDK_TOUCH_END &&
+      event->type != GDK_TOUCH_UPDATE &&
+      event->type != GDK_TOUCH_CANCEL)
+    return GDK_EVENT_PROPAGATE;
+
+  widget = gtk_get_event_widget (event);
+
+  if (!self->use_capture_phase && !should_force_drag (self, widget))
+    return GDK_EVENT_PROPAGATE;
+
+  self->use_capture_phase = TRUE;
+
+  sequence = gdk_event_get_event_sequence (event);
+  retval = gtk_event_controller_handle_event (GTK_EVENT_CONTROLLER (self->touch_gesture), event);
+  state = gtk_gesture_get_sequence_state (self->touch_gesture, sequence);
+
+  if (state == GTK_EVENT_SEQUENCE_DENIED) {
+    gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
+    return GDK_EVENT_PROPAGATE;
+  }
+
+  if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
+    return GDK_EVENT_STOP;
+  } else if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING) {
+    reset (self);
+    return GDK_EVENT_STOP;
+  }
+
+  return retval;
+}
+
 static void
 hdy_swipe_tracker_constructed (GObject *object)
 {
@@ -490,6 +695,17 @@ hdy_swipe_tracker_constructed (GObject *object)
   g_signal_connect_swapped (self->touch_gesture, "drag-end", G_CALLBACK (drag_end_cb), self);
   g_signal_connect_swapped (self->touch_gesture, "cancel", G_CALLBACK (drag_cancel_cb), self);
 
+  g_signal_connect_object (self->swipeable, "event", G_CALLBACK (handle_event_cb), self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->swipeable, "unrealize", G_CALLBACK (reset), self, G_CONNECT_SWAPPED);
+
+  /*
+   * HACK: GTK3 has no other way to get events on capture phase.
+   * This is a reimplementation of _gtk_widget_set_captured_event_handler(),
+   * which is private. In GTK4 it can be replaced with GtkEventControllerLegacy
+   * with capture propagation phase
+   */
+  g_object_set_data (G_OBJECT (self->swipeable), "captured-event-handler", captured_event_cb);
+
   G_OBJECT_CLASS (hdy_swipe_tracker_parent_class)->constructed (object);
 }
 
@@ -504,7 +720,8 @@ hdy_swipe_tracker_dispose (GObject *object)
   if (self->touch_gesture)
     g_signal_handlers_disconnect_by_data (self->touch_gesture, self);
 
-  g_clear_pointer (&self->snap_points, g_free);
+  g_object_set_data (G_OBJECT (self->swipeable), "captured-event-handler", NULL);
+
   g_clear_object (&self->touch_gesture);
   g_clear_object (&self->swipeable);
 
@@ -521,7 +738,7 @@ hdy_swipe_tracker_get_property (GObject    *object,
 
   switch (prop_id) {
   case PROP_SWIPEABLE:
-    g_value_set_object (value, self->swipeable);
+    g_value_set_object (value, hdy_swipe_tracker_get_swipeable (self));
     break;
 
   case PROP_ENABLED:
@@ -596,17 +813,17 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
   object_class->set_property = hdy_swipe_tracker_set_property;
 
   /**
-   * HdySwipeTracker:widget:
+   * HdySwipeTracker:swipeable:
    *
    * The widget the swipe tracker is attached to. Must not be %NULL.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_SWIPEABLE] =
     g_param_spec_object ("swipeable",
                          _("Swipeable"),
                          _("The swipeable the swipe tracker is attached to"),
-                         GTK_TYPE_WIDGET,
+                         HDY_TYPE_SWIPEABLE,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   /**
@@ -615,7 +832,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether the swipe tracker is enabled. When it's not enabled, no events
    * will be processed. Usually widgets will want to expose this via a property.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_ENABLED] =
     g_param_spec_boolean ("enabled",
@@ -630,7 +847,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether to reverse the swipe direction. If the swipe tracker is horizontal,
    * it can be used for supporting RTL text direction.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_REVERSED] =
     g_param_spec_boolean ("reversed",
@@ -645,7 +862,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether to allow dragging with mouse pointer. This should usually be
    * %FALSE.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_ALLOW_MOUSE_DRAG] =
     g_param_spec_boolean ("allow-mouse-drag",
@@ -659,6 +876,69 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
                                     "orientation");
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
+
+  /**
+   * HdySwipeTracker::begin-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @direction: The direction of the swipe
+   * @direct: %TRUE if the swipe is directly triggered by a gesture,
+   *   %FALSE if it's triggered via a #HdySwipeGroup
+   *
+   * This signal is emitted when a possible swipe is detected.
+   *
+   * The @direction value can be used to restrict the swipe to a certain
+   * direction.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_BEGIN_SWIPE] =
+    g_signal_new ("begin-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  2,
+                  HDY_TYPE_NAVIGATION_DIRECTION, G_TYPE_BOOLEAN);
+
+  /**
+   * HdySwipeTracker::update-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @progress: The current animation progress value
+   *
+   * This signal is emitted every time the progress value changes.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_UPDATE_SWIPE] =
+    g_signal_new ("update-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_DOUBLE);
+
+  /**
+   * HdySwipeTracker::end-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @duration: Snap-back animation duration in milliseconds
+   * @to: The progress value to animate to
+   *
+   * This signal is emitted as soon as the gesture has stopped.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_END_SWIPE] =
+    g_signal_new ("end-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_INT64, G_TYPE_DOUBLE);
 }
 
 static void
@@ -671,24 +951,42 @@ hdy_swipe_tracker_init (HdySwipeTracker *self)
 
 /**
  * hdy_swipe_tracker_new:
- * @widget: a #GtkWidget to add the tracker on
+ * @swipeable: a #GtkWidget to add the tracker on
  *
  * Create a new #HdySwipeTracker object on @widget.
  *
  * Returns: the newly created #HdySwipeTracker object
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 HdySwipeTracker *
 hdy_swipe_tracker_new (HdySwipeable *swipeable)
 {
-  g_return_val_if_fail (swipeable != NULL, NULL);
+  g_return_val_if_fail (HDY_IS_SWIPEABLE (swipeable), NULL);
 
   return g_object_new (HDY_TYPE_SWIPE_TRACKER,
                        "swipeable", swipeable,
                        NULL);
 }
 
+/**
+ * hdy_swipe_tracker_get_swipeable:
+ * @self: a #HdySwipeTracker
+ *
+ * Get @self's swipeable widget.
+ *
+ * Returns: (transfer none): the swipeable widget
+ *
+ * Since: 1.0
+ */
+HdySwipeable *
+hdy_swipe_tracker_get_swipeable (HdySwipeTracker *self)
+{
+  g_return_val_if_fail (HDY_IS_SWIPE_TRACKER (self), NULL);
+
+  return self->swipeable;
+}
+
 /**
  * hdy_swipe_tracker_get_enabled:
  * @self: a #HdySwipeTracker
@@ -698,7 +996,7 @@ hdy_swipe_tracker_new (HdySwipeable *swipeable)
  *
  * Returns: %TRUE if @self is enabled
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_enabled (HdySwipeTracker *self)
@@ -716,7 +1014,7 @@ hdy_swipe_tracker_get_enabled (HdySwipeTracker *self)
  * Set whether @self is enabled. When it's not enabled, no events will be
  * processed. Usually widgets will want to expose this via a property.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_enabled (HdySwipeTracker *self,
@@ -745,7 +1043,7 @@ hdy_swipe_tracker_set_enabled (HdySwipeTracker *self,
  *
  * Returns: %TRUE is the direction is reversed
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_reversed (HdySwipeTracker *self)
@@ -763,7 +1061,7 @@ hdy_swipe_tracker_get_reversed (HdySwipeTracker *self)
  * Set whether to reverse the swipe direction. If @self is horizontal,
  * can be used for supporting RTL text direction.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_reversed (HdySwipeTracker *self,
@@ -788,7 +1086,7 @@ hdy_swipe_tracker_set_reversed (HdySwipeTracker *self,
  *
  * Returns: %TRUE is mouse dragging is allowed
  *
- * Since: 0.0.12
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_allow_mouse_drag (HdySwipeTracker *self)
@@ -806,7 +1104,7 @@ hdy_swipe_tracker_get_allow_mouse_drag (HdySwipeTracker *self)
  * Set whether @self can be dragged with mouse pointer. This should usually be
  * %FALSE.
  *
- * Since: 0.0.12
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
@@ -828,136 +1126,54 @@ hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
 }
 
 /**
- * hdy_swipe_tracker_captured_event:
+ * hdy_swipe_tracker_shift_position:
  * @self: a #HdySwipeTracker
- * @event: a captured #GdkEvent
+ * @delta: the position delta
  *
- * Handles an event. This must be called for events received at capture phase
- * only.
+ * Move the current progress value by @delta. This can be used to adjust the
+ * current position if snap points move during the gesture.
  *
- * Returns: %TRUE is the event was handled and must not be propagated
- *
- * Since: 0.0.11
- */
-gboolean
-hdy_swipe_tracker_captured_event (HdySwipeTracker *self,
-                                  GdkEvent        *event)
-{
-  GdkEventSequence *sequence;
-  gboolean retval;
-  GtkEventSequenceState state;
-
-  g_return_val_if_fail (HDY_IS_SWIPE_TRACKER (self), GDK_EVENT_PROPAGATE);
-
-  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
-    return GDK_EVENT_PROPAGATE;
-
-  if (event->type == GDK_SCROLL)
-    return captured_scroll_event (self, event);
-
-  if (event->type != GDK_BUTTON_PRESS &&
-      event->type != GDK_BUTTON_RELEASE &&
-      event->type != GDK_MOTION_NOTIFY &&
-      event->type != GDK_TOUCH_BEGIN &&
-      event->type != GDK_TOUCH_END &&
-      event->type != GDK_TOUCH_UPDATE &&
-      event->type != GDK_TOUCH_CANCEL)
-    return GDK_EVENT_PROPAGATE;
-
-  sequence = gdk_event_get_event_sequence (event);
-  retval = gtk_event_controller_handle_event (GTK_EVENT_CONTROLLER (self->touch_gesture), event);
-  state = gtk_gesture_get_sequence_state (self->touch_gesture, sequence);
-
-  if (state == GTK_EVENT_SEQUENCE_DENIED) {
-    gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
-    return GDK_EVENT_PROPAGATE;
-  }
-
-  if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
-    return GDK_EVENT_STOP;
-  } else if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING) {
-    reset (self);
-    return GDK_EVENT_STOP;
-  }
-
-  return retval;
-}
-
-static gboolean
-is_sorted (gdouble *array,
-           gint     n)
-{
-  gint i;
-
-  if (n < 2)
-    return TRUE;
-
-  for (i = 0; i < n - 1; i++)
-    if (array[i] > array[i + 1])
-      return FALSE;
-
-  return TRUE;
-}
-
-/**
- * hdy_swipe_tracker_confirm_swipe:
- * @self: a #HdySwipeTracker
- * @distance: swipe distance in pixels
- * @snap_points: (array length=n_snap_points) (transfer full): array of snap
- *   points, must be sorted in ascending order
- * @n_snap_points: length of @snap_points
- * @current_progress: initial progress value
- * @cancel_progress: the value that will be used if the swipe is cancelled
- *
- * Confirms a swipe. User has to call this in #HdySwipeTracker::begin signal
- * handler, otherwise the swipe wouldn't start. If there's an animation running,
- * the user should stop it and pass its calue as @current_progress.
- *
- * The call is not a guarantee that the swipe will be started; child widgets
- * may intercept it, in which case #HdySwipeTracker::end will be emitted with
- * @cancel_progress value and 0 duration.
- *
- * @cancel_progress must always be a snap point, or an otherwise a value
- * matching a non-transient state, otherwise the widget will get stuck
- * mid-animation.
- *
- * If there's no animation running, @current_progress and @cancel_progress
- * must be same.
- *
- * Since: 0.0.11
+ * Since: 1.0
  */
 void
-hdy_swipe_tracker_confirm_swipe (HdySwipeTracker *self,
-                                 gdouble          distance,
-                                 gdouble         *snap_points,
-                                 gint             n_snap_points,
-                                 gdouble          current_progress,
-                                 gdouble          cancel_progress)
+hdy_swipe_tracker_shift_position (HdySwipeTracker *self,
+                                  gdouble          delta)
 {
   g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
-  g_return_if_fail (distance > 0.0);
-  g_return_if_fail (snap_points);
-  g_return_if_fail (n_snap_points > 0);
-  g_return_if_fail (is_sorted (snap_points, n_snap_points));
-  g_return_if_fail (current_progress >= snap_points[0]);
-  g_return_if_fail (current_progress <= snap_points[n_snap_points - 1]);
-  g_return_if_fail (cancel_progress >= snap_points[0]);
-  g_return_if_fail (cancel_progress <= snap_points[n_snap_points - 1]);
 
-  if (self->state != HDY_SWIPE_TRACKER_STATE_PREPARING) {
-    gesture_cancel (self);
+  if (self->state != HDY_SWIPE_TRACKER_STATE_PENDING &&
+      self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
     return;
-  }
 
-  if (self->snap_points)
-    g_free (self->snap_points);
-
-  self->distance = distance;
-  self->initial_progress = current_progress;
-  self->progress = current_progress;
-  self->velocity = 0;
-  self->snap_points = snap_points;
-  self->n_snap_points = n_snap_points;
-  self->cancel_progress = cancel_progress;
-  self->state = HDY_SWIPE_TRACKER_STATE_PENDING;
+  self->progress += delta;
+  self->initial_progress += delta;
+}
+
+void
+hdy_swipe_tracker_emit_begin_swipe (HdySwipeTracker        *self,
+                                    HdyNavigationDirection  direction,
+                                    gboolean                direct)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_BEGIN_SWIPE], 0, direction, direct);
+}
+
+void
+hdy_swipe_tracker_emit_update_swipe (HdySwipeTracker *self,
+                                     gdouble          progress)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_UPDATE_SWIPE], 0, progress);
+}
+
+void
+hdy_swipe_tracker_emit_end_swipe (HdySwipeTracker *self,
+                                  gint64           duration,
+                                  gdouble          to)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_END_SWIPE], 0, duration, to);
 }
